
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <iup.h>
#include <iupval.h>
#include <iupgc.h>
#include <im.h>
#include <math.h>
#include <stdarg.h>

#include "main.h"
#include "image.h"
#include "dither.h"
#include "warp.h"
#include "list.h"
#include "filter.h"
#include "painter.h"
#include "gauge.h"

#define PI 3.141592653589f

int fTrue (p_image in)
{
	return 1;
}

int fTestMono (p_image in)
{
	return (in->type == IMAGE_MONO);
}

int fTestRGB (p_image in)
{
	return (in->type == IMAGE_RGB);
}

/*--------------------------------------------------------------------*\
* Filters
* Input
*   in: input image
*   out: output image
* Returns:
*   0 in case of error, 1 if sucessfull
\*--------------------------------------------------------------------*/

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

#define MAX(a,b) ((a)>(b)?(a):(b))

static char *string(const char *format, ...)
{
	static char buffer[1025];
    va_list ap;
    va_start(ap, format);
    vsprintf(buffer,format, ap);
    va_end(ap);
	return buffer;
}

static int save_rgb(p_image in, char *file)
{
    uchar *red, *green, *blue;
    int width, height;
	int ret = 0;
    if (!image_splitnewrgb(in, &red, &green, &blue, &width, &height))
		return 0;
    if(imSaveRGB(width, height, IM_RAS, blue, green, red, file) != IM_ERR_NONE)
		ret = 0;
	free(red); free(green); free(blue);
	return ret;
}

static int save_mono(p_image in, char *file)
{
    uchar *alpha;
    int width, height;
	int ret = 0;
    if (!image_splitnewmono(in, &alpha, &width, &height))
		return 0;
    if(imSaveRGB(width, height, IM_RAS, alpha, alpha, alpha, file) != IM_ERR_NONE)
		ret = 0;
	free(alpha);
	return ret;
}

static int getstrokes(p_image input, p_image area, p_list *list, int S, 
	int factor, int level, int n)
{
	int count = 0;
	uchar *buffer;
	t_image position, inputscaled, areascaled, I;
	int x, y;
	long int skip;
	double s, p;

	main_status("Generating stroke positions image...");
	warp_scale(area, area->width/factor, area->height/factor, &areascaled);
	image_copy(&areascaled, &position);
	
	//s = 20.0;	p = 3.0;
	/*s = 20.0f*S/factor;
	p = 6.0f/level;*/

	/*****************************/
	/* PARTE QUE PRECISA AJUSTAR */
	if (n > 1)
	{
		double minp = 2.0;
		double maxp = 6.5;

		/*s = (S - input->width)*level;
		s += n * input->width - S;
		s /= n - 1;*/

		s = 2 * input->width * pow((double)S / (2 * input->width), (double)(level - 1) / (n - 1));

		p = (minp - maxp) * level;
		p += maxp * (n - 1) + maxp - minp;
		p /= n - 1;
	}
	else
	{
		s = S;
		p = 3.5;
	}

	dither_mono(&position, s, p);
	save_mono(&position, string("points%d.ras", level));
	warp_scale(input, input->width/factor, input->height/factor, &inputscaled);

	buffer = position.buffer;
	skip = (position.row - position.width)*position.type;
	gauge_setmax(position.height * position.width);
	gauge_start();

	/* aloca buffer esperto */
	image_create(S, S, IMAGE_MONO, &I);

	for (y = 0; y < position.height; y++) {
		for (x = 0; x < position.width; x++) {
			if (!(*buffer)) {
				int xc, yc;
				float w, l;
				double theta;
				uchar bgr[3];
				painter_strokepar(&inputscaled, x, y, S, &xc, &yc, &theta, &w, &l, bgr, &I);
				list_prepend(list, factor*xc, factor*yc, factor*w, factor*l, (float) theta, bgr, level);
				count++;
			}
			buffer++;
		}
		gauge_increase(position.width);
		gauge_print();
		buffer += skip;
	}
	gauge_end();
	image_kill(&inputscaled);
	image_kill(&areascaled);
	image_kill(&position);

	main_status("Collected %d strokes...", count);
	return count;
}

static int cbPainterBack (Ihandle *self)
{
	p_image in, out;
	int clear;

	/* clear image or not */
	clear = !strcmp(IupGetAttribute(IupGetAttribute(self, "PAINTER_CLEAR"), IUP_VALUE), IUP_ON);

	/* get input image */
	in = (p_image)IupGetAttribute(self, "PAINTER_IMAGE_IN");
	assert (in);

	/* get output image */
	out = (p_image)IupGetAttribute(self, PAINTER_IMAGE);
	assert (out);

	/* clear image to begin composition from the start */
	if (clear)
		image_clear(out, 255);
	else
		image_copy(in, out);

	/* set the current stroke count to 0 */
	IupSetfAttribute(IupGetAttribute(self, "PAINTER_STROKE_COUNTER"), IUP_VALUE, "%d", 0);

	/* update screen */
	main_repaint(IupGetDialog(self));

	return IUP_DEFAULT;
}

static int composition (Ihandle *self, int repaint)
{
	p_image in, out, stroke;
	p_list strokes;
	int clear;
	char levels[512];
	static int running = 0;
	int count, total;

	if (running)
	{
		running = 0;
		IupStoreAttribute(IupGetAttribute(self, "PAINTER_PLAY"), IUP_TITLE, ">");
		return IUP_DEFAULT;
	}

	/* resolution levels */
	strcpy(levels, IupGetAttribute(IupGetAttribute(self, "PAINTER_LEVELS"), IUP_VALUE));

	/* clear image or not */
	clear = !strcmp(IupGetAttribute(IupGetAttribute(self, "PAINTER_CLEAR"), IUP_VALUE), IUP_ON);

	/* get the current stroke count */
	count = IupGetInt(IupGetAttribute(self, "PAINTER_STROKE_COUNTER"), IUP_VALUE);
	total = IupGetInt(IupGetAttribute(self, "PAINTER_NSTROKES"), IUP_TITLE);

	/* if the counter is in the end, reestart from the beggining */
	if (count == total)
	{
		cbPainterBack(self);
		count = 0;
	}

	/* get input image */
	in = (p_image)IupGetAttribute(self, "PAINTER_IMAGE_IN");
	assert (in);

	/* get output image */
	out = (p_image)IupGetAttribute(self, PAINTER_IMAGE);
	assert (out);

	/* get stroke image */
	stroke = (p_image)IupGetAttribute(self, "PAINTER_STROKE");
	assert (stroke);

	/* get list of strokes */
	strokes = (p_list)IupGetAttribute(self, "PAINTER_STROKES");
	assert (strokes);

	/* compose image */
	main_status("Creating painterly image...");
	gauge_start();
	{
		int xc, yc, w, l, level;
		uchar bgr[3];
		float theta;
		int i = 1;
		p_list p = strokes;

		running = 1;
		IupStoreAttribute(IupGetAttribute(self, "PAINTER_PLAY"), IUP_TITLE, "| |");

		while ((p = list_enum(p, &xc, &yc, &w, &l, &theta, bgr, &level))) {
			t_image scaled;
			t_image rotated;

			i++;
			if (i > count)
			{
				/* if level is not checked, don't paint it */
				if (levels[level-1] == '+')
				{
					warp_scale(stroke, l, w, &scaled);
					warp_rotatemono(&scaled, theta, &rotated);
					image_blendrgb(out, &rotated, bgr, xc, yc);
					image_kill(&scaled);
					image_kill(&rotated);
				
					/* update screen */
					if (repaint)
						main_repaint(IupGetDialog(self));
				}
				if (!running)
					break;

				/* update counter */
				IupSetfAttribute(IupGetAttribute(self, "PAINTER_STROKE_COUNTER"), IUP_VALUE, "%d", i);
				IupLoopStep();
			}

			gauge_increase(1);
			gauge_print();
		}
		running = 0;
		IupStoreAttribute(IupGetAttribute(self, "PAINTER_PLAY"), IUP_TITLE, ">");
	}
	gauge_end();

	/* update screen */
	if (repaint)
		main_repaint(IupGetDialog(self));

	return IUP_DEFAULT;
}

static int cbPainterPlay (Ihandle *self)
{
	return composition(self, 1);
}

int cbPainterNStroke(Ihandle *self, int c, char *after)
{
	p_image in, out, stroke;
	p_list strokes;
	int count;
	int clear;
	char levels[512];

	if (!sscanf(after, "%d", &count))
	{
		count = 0;
		return IUP_IGNORE;
	}

	/* resolution levels */
	strcpy(levels, IupGetAttribute(IupGetAttribute(self, "PAINTER_LEVELS"), IUP_VALUE));

	/* clear image or not */
	clear = !strcmp(IupGetAttribute(IupGetAttribute(self, "PAINTER_CLEAR"), IUP_VALUE), IUP_ON);

	cbPainterBack(self);

	/* get input image */
	in = (p_image)IupGetAttribute(self, "PAINTER_IMAGE_IN");
	assert (in);

	/* get output image */
	out = (p_image)IupGetAttribute(self, PAINTER_IMAGE);
	assert (out);

	/* get stroke image */
	stroke = (p_image)IupGetAttribute(self, "PAINTER_STROKE");
	assert (stroke);

	/* get list of strokes */
	strokes = (p_list)IupGetAttribute(self, "PAINTER_STROKES");
	assert (strokes);

	/* compose image */
	main_status("Creating painterly image...");
	gauge_start();
	{
		int xc, yc, w, l, level;
		uchar bgr[3];
		float theta;
		int i = 1;
		p_list p = strokes;

		while ((p = list_enum(p, &xc, &yc, &w, &l, &theta, bgr, &level))) {
			t_image scaled;
			t_image rotated;

			if (i <= count)
			{
				/* if level is not checked, don't paint it */
				if (levels[level-1] == '+')
				{
					warp_scale(stroke, l, w, &scaled);
					warp_rotatemono(&scaled, theta, &rotated);
					image_blendrgb(out, &rotated, bgr, xc, yc);
					image_kill(&scaled);
					image_kill(&rotated);
				}
			}

			i++;
			gauge_increase(1);
			gauge_print();
		}
	}
	gauge_end();

	/* update counter */
	IupSetfAttribute(IupGetAttribute(self, "PAINTER_STROKE_COUNTER"), IUP_VALUE, "%d", count);
	IupSetfAttribute(IupGetAttribute(self, "PAINTER_STROKE_COUNTER"), IUP_CARET, "%d", 10);

	/* update screen */
	main_repaint(IupGetDialog(self));

	return IUP_DEFAULT;
}

int fPainter (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	int i, n;
	char *d[256];
	char attr[512];

	Ihandle *images;
	Ihandle *dialog;
	Ihandle *multi;
	Ihandle *sh;
	Ihandle *stroke_counter, *nstrokes, *levels, *clear, *play;

	t_image area;
	p_image stroke;
	p_list strokes;

	int s;
	int count;
	int factor, size, level;

	/* list of available images to use as stroke */
	images = IupList("stroke_CB");
	n = IupGetAllDialogs(d, 256);
	for (i = 0; i < n; i++)
	{
		sprintf (attr, "%d", i+1);
		IupStoreAttribute(images, attr, d[i]);
	}

	/* parameters dialog */
	dialog = IupDialog(
		IupVbox(
			IupSetAttributes(IupHbox(
			IupSetAttributes(IupFill(), "SIZE=5"),
				IupLabel("s = "),
				IupSetAttributes(IupFill(), "SIZE=5"),
				(sh = IupSetAttributes(IupText("null_CB"), "VALUE=15, EXPAND=HORIZONTAL")),
				NULL
			), "ALIGNMENT=ACENTER"),
			IupSetAttributes(IupFill(), "SIZE=5"),
			IupLabel("Stroke image: "),
			images, 
			IupSetAttributes(IupFill(), "SIZE=5"),
			IupSetAttributes(multi = IupToggle("Multi-resolution", "null_CB"), "EXPAND=HORIZONTAL, VALUE=ON"),
			IupSetAttributes(IupFill(), "SIZE=5"),
			NULL
		)
	);

	IupPopup(dialog, IUP_CENTER, IUP_CENTER);

	/* get stroke image */
	stroke = (p_image)IupGetAttribute(IupGetHandle(IupGetAttribute(images, IupGetAttribute(images, IUP_VALUE))), PAINTER_IMAGE);
	if (!stroke || stroke->type != IMAGE_MONO)
		return 0;

	/* stroke area image */
	main_status("Calculating stroke area image...");
	s = IupGetInt(sh, IUP_VALUE);
	painter_strokearea(in, s, &area);

	/* stroke list creation */
	strokes = list_create();

	count = 0;
	factor = 1;
	level = 1;

	if (!strcmp(IupGetAttribute(multi, IUP_VALUE), IUP_ON))
	{
		/* multiresolution algorithm*/

		size = MAX(in->width, in->height);
		n = (int)(log((double)(size) / (double)(s)) / log(2.0) - 1);
		if (n < 1) n = 1;
	}
	else
	{
		/* paper algorithm */

		size = 4*s;
		n = 1;
	}

	do {
		main_status("Processing level %d...", level);
		count += getstrokes(in, &area, &strokes, s, factor, level, n);
		size /= 2;
		factor *= 2;
		level++;
	} while (size > 4*s);

	/* sort by stroke size */
	list_mergesort(&strokes);

	image_kill(&area);

	*extrabox = IupSetAttributes(IupHbox(
		IupLabel("Strokes: "),
		IupSetAttributes(stroke_counter = IupText("PainterNStroke_CB"), "VALUE=0, SIZE=40x"),
		IupLabel(" / "),
		IupSetAttributes(nstrokes = IupLabel(""), "SIZE=40x"),
		IupSetAttributes(IupFill(), "SIZE=5"),
		IupSetAttributes(levels = IupList("null_CB"), "MULTIPLE=YES"),
		IupSetAttributes(IupFill(), "SIZE=5"),
		IupSetAttributes(clear = IupToggle("Clear", "null_CB"), "VALUE=ON"),
		IupSetAttributes(IupFill(), "SIZE=5"),
		IupSetAttributes(IupButton("|<<", "PainterBack_CB"), "SIZE=30x"),
		IupSetAttributes(play = IupButton(">", "PainterPlay_CB"), "SIZE=30x"),
		NULL
	), "ALIGNMENT=ACENTER");

	/* resolution levels */
	for (i = 1; i < level; i++)
	{
		char item[32];
		sprintf(item, "%d", i);
		IupStoreAttribute(levels, item, item);
		attr[i-1] = '+';
	}
	attr[level-1] = '\0';
	IupStoreAttribute(levels, IUP_VALUE, attr);
	IupSetfAttribute(nstrokes, IUP_TITLE, "%d", count);

	IupSetAttribute(*extrabox, "PAINTER_STROKE_COUNTER", (char*)stroke_counter);
	IupSetAttribute(*extrabox, "PAINTER_NSTROKES", (char*)nstrokes);
	IupSetAttribute(*extrabox, "PAINTER_STROKE", (char*)stroke);
	IupSetAttribute(*extrabox, "PAINTER_STROKES", (char*)strokes);
	IupSetAttribute(*extrabox, "PAINTER_IMAGE_IN", (char*)in);
	IupSetAttribute(*extrabox, "PAINTER_LEVELS", (char*)levels);
	IupSetAttribute(*extrabox, "PAINTER_CLEAR", (char*)clear);
	IupSetAttribute(*extrabox, "PAINTER_PLAY", (char*)play);
	IupSetAttribute(*extrabox, PAINTER_IMAGE, (char*)out);
	
	/* number of strokes edit box callback */
	IupSetFunction("PainterNStroke_CB", cbPainterNStroke);

	/* play button callback */
	IupSetFunction("PainterPlay_CB", cbPainterPlay);
	IupSetFunction("PainterBack_CB", cbPainterBack);

	/* need to kill list of strokes */
	//list_kill(&strokes);

	/* image composition */
	image_kill(out);
	image_create(in->width, in->height, IMAGE_RGB, out);
	gauge_setmax(count);
	composition(*extrabox, 0);

	return 1;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fStrokeArea (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	int s = 15;
	char *fmt =
	{
		"Stroke area image\n"
		" s = %20.10%d\n"
	};

	*extrabox = NULL;

	/* only RGB images */
	if (in->type != IMAGE_RGB)
		return 0;

	if (IupScanf (fmt, &s) == -1)
		return 0;

	if (! painter_strokearea(in, s, out))
		return 0;

	sprintf(description, "Stroke area [%d]", s);
	return 1;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fColorDiff (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	uchar bgr[4];
	bgr[0] = 0x00;
	bgr[1] = 0x00;
	bgr[2] = 0x00;
	bgr[3] = 0x00;

	*extrabox = NULL;

	if (! IupGetColor(IUP_CENTER, IUP_CENTER, bgr+2, bgr+1, bgr))
		return 0;

	if (!image_create(in->width, in->height, IMAGE_MONO, out))
		return 0;

	if (! painter_colordiff(in, bgr, out))
		return 0;

	sprintf(description, "Color difference [%d %d %d]", (int)bgr[2], (int)bgr[1], (int)bgr[0]);
	return 1;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fScale (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	t_image temp;
	int w = in->width / 2;
	int h = in->height / 2;

	char *fmt =
	{
		"Scale\n"
		" width = %20.10%d\n"
		" height = %20.10%d\n"
	};

	*extrabox = NULL;

	if (IupScanf (fmt, &w, &h) == -1)
		return 0;

	if (! warp_scale(in, w, h, &temp))
		return 0;

	if (! image_copy(&temp, out))
		return 0;

	image_kill(&temp);

	sprintf(description, "Scale [%dx%d]", w, h);
	return 1;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fRotate (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	t_image temp;
	float ang = 45.0f;
	char *fmt =
	{
		"Rotation\n"
		" angle = %20.10%f\n"
	};

	*extrabox = NULL;

	/* only mono images */
	if (in->type != IMAGE_MONO)
		return 0;

	if (IupScanf (fmt, &ang) == -1)
		return 0;

	if (! warp_rotatemono(in, PI * ang / 180.0f, &temp))
		return 0;

	if (! image_copy(&temp, out))
		return 0;

	image_kill(&temp);

	sprintf(description, "Rotate [%.2f]", ang);
	return 1;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fCopy (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	if (! image_copy(in, out))
		return 0;

	*extrabox = NULL;

	sprintf(description, "Copy");
	return 1;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int cbDitherVal(Ihandle *self, double val)
{
	double s, p;
	Ihandle *vals, *valp;
	p_image in, out;
	
	/* parameters valuators */
	vals = IupGetAttribute(self, "PAINTER_VALS");
	valp = IupGetAttribute(self, "PAINTER_VALP");
	assert(vals);
	assert(valp);

	/* images */
	in = (p_image)IupGetAttribute(self, "PAINTER_IMAGE_IN");
	assert (in);

	out = (p_image)IupGetAttribute(self, "PAINTER_IMAGE_OUT");
	assert (out);

	/* Dithering parameters values */
	s = IupGetFloat(vals, IUP_VALUE);
	p = IupGetFloat(valp, IUP_VALUE);

	if (!image_copy(in, out))
		return IUP_IGNORE;

	if (!dither_mono(out, s, p))
		return IUP_IGNORE;

	/* change value of the labels */
	IupSetfAttribute(IupGetHandle("PAINTER_S"), IUP_TITLE, "%.0f", s);
 	IupSetfAttribute(IupGetHandle("PAINTER_P"), IUP_TITLE, "%.0f", p);

	/* refresh screen */
	main_repaint(IupGetDialog(self));

	return 1;
}

int fDither (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	Ihandle *vals, *valp;
	Ihandle *output = NULL;
	Ihandle *hs, *hp;
	int s = 50;
	int p = 3;
	char *fmt =
	{
		"Dithering\n"
		" s = %20.10%d\n"
		" p = %20.10%d\n"
	};

	/* only mono images */
	if (in->type != IMAGE_MONO)
		return 0;

	if (IupScanf (fmt, &s, &p) == -1)
		return 0;

	/* output goes to another image */
	/* so, we can do it interactively */

	/* Valuator for the dithering parameters */
	vals = IupVal(ICTL_HORIZONTAL);
	valp = IupVal(ICTL_HORIZONTAL);
  
	IupSetfAttribute(vals, IUP_MIN, "%d", 1);
	IupSetfAttribute(vals, IUP_MAX, "%d", 20*s);
	IupSetfAttribute(vals, IUP_VALUE, "%d", s);
	IupSetfAttribute(vals, IUP_SIZE, "%dx", 50);

	IupSetfAttribute(valp, IUP_MIN, "%d", 1);
	IupSetfAttribute(valp, IUP_MAX, "%d", 15);
	IupSetfAttribute(valp, IUP_VALUE, "%d", p);
	IupSetfAttribute(valp, IUP_SIZE, "%dx", 50);

	/* callbacks */
	IupSetAttribute(vals, ICTL_MOUSEMOVE_CB, "DitherVal_CB");
	IupSetAttribute(valp, ICTL_MOUSEMOVE_CB, "DitherVal_CB");
	IupSetFunction("DitherVal_CB", cbDitherVal);

	/* Hbox with the dithering parameters */
	*extrabox = IupSetAttributes(IupHbox(
		IupFill(),
		IupLabel("s: "),
		(hs = IupLabel("0000")),
		vals,
		IupFill(),
		IupLabel("p: "),
		(hp = IupLabel("0000")),
		valp,
		IupFill(),
		NULL
	), "ALIGNMENT=ACENTER");
	IupSetHandle("PAINTER_S", hs);
	IupSetHandle("PAINTER_P", hp);

	/* stores images in the dialog */
	IupSetAttribute(*extrabox, "PAINTER_IMAGE_IN", (char*)in);
	IupSetAttribute(*extrabox, "PAINTER_IMAGE_OUT", (char*)out);

	/* stores two IupVal's in the dialog */
	IupSetAttribute(*extrabox, "PAINTER_VALS", (char*)vals);
	IupSetAttribute(*extrabox, "PAINTER_VALP", (char*)valp);

	if (!image_copy(in, out))
		return 0;

	if (!dither_mono(out, s, p))
		return 0;

	sprintf(description, "Dithering [%d, %d]", s, p);

	return 1;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fRed (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	uchar *r, *g, *b;
	int w, h;
	int ret;

	*extrabox = NULL;

	if (! image_splitnewrgb(in, &r, &g, &b, &w, &h))
		return 0;

	ret = image_mergenewmono(r, w, h, out);
	free(r); free(g); free(b);

	sprintf(description, "Red channel");
	return ret;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fGreen (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	uchar *r, *g, *b;
	int w, h;
	int ret;

	*extrabox = NULL;

	if (! image_splitnewrgb(in, &r, &g, &b, &w, &h))
		return 0;

	ret = image_mergenewmono(g, w, h, out);
	free(r); free(g); free(b);

	sprintf(description, "Green channel");
	return ret;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fBlue (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	uchar *r, *g, *b;
	int w, h;
	int ret;

	*extrabox = NULL;

	if (! image_splitnewrgb(in, &r, &g, &b, &w, &h))
		return 0;

	ret = image_mergenewmono(b, w, h, out);
	free(r); free(g); free(b);

	sprintf(description, "Blue channel");
	return ret;
}

/*--------------------------------------------------------------------*\
\*--------------------------------------------------------------------*/

int fGray (p_image in, p_image out, char *description, Ihandle **extrabox)
{
	uchar *r, *g, *b;
	uchar *pr, *pg, *pb;
	int w, h;
	int i, j;
	int ret;
	long skip = (in->row - in->width) * in->type;

	*extrabox = NULL;

	if (! image_splitnewrgb(in, &r, &g, &b, &w, &h))
		return 0;

	pr = r; pg = g; pb = b;
	for (i = 0; i < h; i++)
	{
		for (j = 0; j < w; j++)
		{
			*pr = ((*pr) + (*pb) + (*pb)) / 3;
			pr++;
			pg++;
			pb++;
		}
	}

	ret = image_mergenewmono(r, w, h, out);
	free(r); free(g); free(b);

	sprintf(description, "Grayscale");
	return ret;
}

