
#include <stdlib.h>
#include <windows.h>
#include <cd.h>
#include <cdiup.h>
#include <cdnative.h>
#include <im.h>
#include <iup.h>
#include <iupgauge.h>
#include <iuptabs.h>
#include <iupgc.h>
#include <iupval.h>
#include <iupcpi.h>
#include <assert.h>
#include <stdio.h>

#include "main.h"
#include "gauge.h"
#include "warp.h"
#include "image.h"
#include "filter.h"
#include "painter.h"

static Ihandle* dlgMain (void);
static Ihandle* dlgImage (const char *title, p_image im, Ihandle *extrabox);
static Ihandle* mnPopup (void);
static Ihandle* _main_dialog = NULL;
static Ihandle* _status = NULL;

/*--------------------------------------------------------------------*\
* Function to print a status message
\*--------------------------------------------------------------------*/

void main_status(const char *format, ...)
{
	static char buffer[1025];
	va_list ap;
	va_start(ap, format);
	vsprintf(buffer, format, ap);
	va_end(ap);

	printf ("%s\n", buffer);
	IupSetfAttribute(_status, IUP_VALUE, "%s\n%s", buffer, IupGetAttribute(_status, IUP_VALUE));
	IupLoopStep();
}

void main_status_image(Ihandle *self, const char *format, ...)
{
	static char buffer[1025];
	va_list ap;
	va_start(ap, format);
	vsprintf(buffer, format, ap);
	va_end(ap);

	printf ("%s\n", buffer);
	IupStoreAttribute(IupGetAttribute(self, PAINTER_STATUS), IUP_TITLE, buffer);
	IupLoopStep();
}

/*--------------------------------------------------------------------*\
* Function to blit an image, by Diego
\*--------------------------------------------------------------------*/

int put_image(Ihandle *canvas, p_image im)
{
	HDC dc;
	HWND conid;
	BITMAPINFOHEADER bi = {
		sizeof(bi),
		im->row,
		im->height,
		1,
		24,
		BI_RGB,
		0,
		1000, 1000,
		0,
		0
	};
	if (im->reference)
		return 0;

	if (im->type != IMAGE_RGB)
		return 0;

	conid = (HWND) IupGetAttribute(canvas, IUP_WID);
	dc = GetDC(conid);
	SetDIBitsToDevice(dc, 0, 0, im->width, im->height, 0, 0, 0, im->height, im->buffer, (BITMAPINFO*)&bi, 0);
	GdiFlush();
	return 1;
}

/*--------------------------------------------------------------------*\
* Utilities to load and save, using IM
\*--------------------------------------------------------------------*/

static int load(char *file, p_image out)
{
	uchar *red, *green, *blue;
	long color[256];
	int width, height, type, pal_size, format;

	if (imFileFormat(file, &format) != IM_ERR_NONE)
		return 0;

	imImageInfo(file, &width, &height, &type, &pal_size);

	red   = malloc(width * height);
	green = malloc(width * height);
	blue  = malloc(width * height);

	if (!red || !green || !blue) {
		if (red)   free(red);
		if (green) free(green);
		if (blue)  free(blue);
		return 0;
	}

	switch (type)
	{
	case IM_MAP:
		imLoadMap(file, red, color);
		image_mergenewmono(red, width, height, out);
		break;

	case IM_RGB:
		imLoadRGB(file, red, green, blue);
		//image_mergenewrgb(red, green, blue, width, height, out);
		image_mergenewrgb(blue, green, red, width, height, out);
		break;
	}

	free(red); free(green); free(blue);

	return 1;
}

static int save(p_image in, char *file)
{
	uchar *red, *green, *blue, *alpha;
	int width, height;
	int ret = 1;

	switch(in->type)
	{
	case IMAGE_RGB:
		if (!image_splitnewrgb(in, &red, &green, &blue, &width, &height))
			return 0;

		if (imSaveRGB(width, height, IM_JPG|IM_COMPRESSED, red, green, blue, file) != IM_ERR_NONE)
			ret = 0;

		free(red); free(green); free(blue);
		break;

	case IMAGE_MONO:
		if (!image_splitnewmono(in, &alpha, &width, &height))
			return 0;

		if (imSaveRGB(width, height, IM_JPG|IM_COMPRESSED, alpha, alpha, alpha, file) != IM_ERR_NONE)
			ret = 0;

		free(alpha);
		break;
	}

	return ret;
}

/*--------------------------------------------------------------------*\
* Opening an image, creates a new dialog
* Input
*   self: handle of the popup menu
\*--------------------------------------------------------------------*/
static int cbOpen (Ihandle* self)
{
	Ihandle *filedlg = NULL; 
	Ihandle *dialog = NULL;
	int result = 0; 
	char *filename = NULL; 
	p_image im;

	/* creates the structure */
	im = (p_image) malloc (sizeof(t_image));

	/* ask for a filename */
	filedlg = IupFileDlg(); 

	IupSetAttribute(filedlg, IUP_DIALOGTYPE, IUP_OPEN); 
	IupSetAttribute(filedlg, IUP_TITLE, "Open..."); 
	IupSetAttribute(filedlg, IUP_FILTER, "*.*"); 
	IupSetAttribute(filedlg, IUP_FILTERINFO, "All files"); 

	IupPopup(filedlg, IUP_CENTER, IUP_CENTER); 

	filename = IupGetAttribute(filedlg, IUP_VALUE); 
	if (!filename) 
		return IUP_IGNORE;

	/* open the image */
	main_status("Opening %s", filename);
	if (!load(filename, im))
	{
		IupMessage("Error", "Error opening image!");
		return IUP_IGNORE;
	}

	/* create and show a new dialog */
   dialog = dlgImage(filename, im, NULL);
   IupShowXY(dialog, IUP_CENTER, IUP_CENTER);

	return IUP_DEFAULT;
}

/*--------------------------------------------------------------------*\
* Saving an image to a file
* Input
*   self: handle of the popup menu
\*--------------------------------------------------------------------*/
static int cbSave (Ihandle* self)
{
	p_image im;
	Ihandle *canvas;
	Ihandle *filedlg = NULL; 
	int result = 0; 
	char *filename = NULL; 

	filedlg = IupFileDlg(); 

	IupSetAttribute(filedlg, IUP_DIALOGTYPE, IUP_SAVE); 
	IupSetAttribute(filedlg, IUP_TITLE, "Save as..."); 
	IupSetAttribute(filedlg, IUP_FILTER, "*.jpg"); 
	IupSetAttribute(filedlg, IUP_FILTERINFO, "JPEG Files (*.jpg)"); 

	IupPopup(filedlg, IUP_CENTER, IUP_CENTER); 

	filename = IupGetAttribute(filedlg, IUP_VALUE); 
	if (!filename) 
		return IUP_IGNORE;

	/* get canvas and image */
	canvas = IupGetAttribute(self, PAINTER_CANVAS);
	im = (p_image)IupGetAttribute(canvas, PAINTER_IMAGE);

	main_status("Saving %s", filename);

	if (!save(im, filename))
		IupMessage("Error", "Error saving image");

	return IUP_DEFAULT;
}

/*--------------------------------------------------------------------*\
* Closing an image
* Input
*   self: handle of the popup menu
\*--------------------------------------------------------------------*/
static int cbClose (Ihandle* self)
{
	Ihandle *dialog;
	Ihandle *iupcanvas;
	cdCanvas *canvas;
	p_image im;

	/* get cdCanvas from the popup menu */
	iupcanvas = IupGetAttribute(self, PAINTER_CANVAS);
	assert (iupcanvas);

	/* destroy the image */
	im = (p_image)IupGetAttribute(iupcanvas, PAINTER_IMAGE);
	assert(im);
	image_kill(im);
	free(im);

	/* destroy the canvas */
	canvas = (cdCanvas*)IupGetAttribute(iupcanvas, "_CD_CANVAS");
	assert(canvas);
	cdKillCanvas(canvas);

	dialog = IupGetDialog(iupcanvas);
	
	/* unregister the handle */
	IupSetHandle(NULL, dialog);

	/* destroy the dialog */
	IupHide(dialog);
	IupDestroy(dialog);

	return IUP_DEFAULT;
}

/*--------------------------------------------------------------------*\
* Exit door
\*--------------------------------------------------------------------*/
static int cbExit (Ihandle* self)
{
	return IUP_CLOSE;
}

/*--------------------------------------------------------------------*\
* About dialog
\*--------------------------------------------------------------------*/
static int cbAbout (Ihandle* self)
{
	char msg[2048];
	sprintf (msg, "%s - %s\n\n%s\n%s\n%s\n\nIUP %s\nCD %s\nIM %s", 
		"Painter",
		"Nov/2000",
		"Danilo Tuler - tuler@tecgraf.puc-rio.br",
		"Diego Nehab - diego@tecgraf.puc-rio.br", 
		"Diogo Andrade - diogo@tecgraf.puc-rio.br",
		IupGetGlobal(IUP_VERSION),
		cdVersion(),
		imVersion());

	IupMessage ("About", msg);

	return IUP_DEFAULT;
}

/*--------------------------------------------------------------------*\
* Enable or disable popup menu item
* Input
*   menu: handle of the popup menu
\*--------------------------------------------------------------------*/
static int configMenu (Ihandle *menu, p_image im)
{
	Ihandle *item;
	FILTERCHECK *f;

	item = iupGetFirstChild(menu);
	while (item)
	{
		f = (FILTERCHECK*)IupGetAttribute(item, PAINTER_FILTERCHECK);
		if (f)
		{
			if ((*f)(im))
				IupSetAttribute(item, IUP_ACTIVE, IUP_YES);
			else
				IupSetAttribute(item, IUP_ACTIVE, IUP_NO);
		}
		item = iupGetBrother(item);
	}

	return 1;
}

/*--------------------------------------------------------------------*\
* Click over an image canvas
* Input
*   self: handle of the canvas
*   b: button 
*   e: 
*   x: position of the mouse
*   y: position of the mouse
*   r: 
\*--------------------------------------------------------------------*/
static int cbClick(Ihandle* self, int b, int e, int x, int y, char* r)
{
	Ihandle *im = NULL;
	Ihandle *popup = NULL;
	Ihandle *output = NULL;

	/* destroi um menu criado anteriormente */
	popup = IupGetHandle (PAINTER_POPUP);
	if (popup) IupDestroy(popup);

	/* cria um novo menu */
	popup = mnPopup();
	assert (popup);

	im = IupGetAttribute(self, PAINTER_IMAGE);
	assert (im);

	switch (b)
	{
	/* left click, do nothing */
	case IUP_BUTTON1:
		break;

	/* right click, popups the context menu */
	case IUP_BUTTON3:
		configMenu(popup, im);
		IupSetAttribute(popup, PAINTER_CANVAS, self);
		IupPopup(popup, IUP_MOUSEPOS, IUP_MOUSEPOS);
		break;
	}
	return IUP_DEFAULT;
}


/*--------------------------------------------------------------------*\
* Motion over the canvas
* Input
*   self: handle of the canvas
*   x: position of the mouse
*   y: position of the mouse
*   r: status of the button
\*--------------------------------------------------------------------*/
int cbMotion(Ihandle *self, int x, int y, char *r)
{
	p_image im = NULL;
	int xc, yc;
	double theta;
	float w, l;
	uchar bgr[3];
	t_image I;
	int s = 15;

	/* get image from the IUP canvas */
	im = (p_image)IupGetAttribute(self, PAINTER_IMAGE);
	assert(im);

	image_create(s, s, IMAGE_MONO, &I);

	painter_strokepar(im, x, y, s, &xc, &yc, &theta, &w, &l, bgr, &I);
	main_status_image(self, "(%d,%d) - (%d,%d) - [%.0fx%.0f] - %.4f rad", x, y, xc, yc, w, l, theta);

	return IUP_DEFAULT;
}

/*--------------------------------------------------------------------*\
* Repaint of the canvas, draws the image
* Input
*   self: handle of the canvas
*   x: position of the scrollbar
*   y: position of the scrollbar
\*--------------------------------------------------------------------*/
static int cbRepaint(Ihandle* self, float x, float y)
{
	cdCanvas *canvas;
	p_image im = NULL;

	/* get cdCanvas from the IUP canvas */
	canvas = (cdCanvas*)IupGetAttribute(self, "_CD_CANVAS");
	assert(canvas);

	/* get image from the IUP canvas */
	im = (p_image)IupGetAttribute(self, PAINTER_IMAGE);
	assert(im);

	cdActivate(canvas);

	/* uninterlace image */
	switch (im->type)
	{
	case IMAGE_MONO:
		cdPutImageRectMap(im->row, im->height, im->buffer, NULL, 0, 0, 0, 0, 0, 0, 0, 0);
		break;

	case IMAGE_RGB:
		put_image(self, im);
		break;

	default:
		assert(0);
		break;
	}

	return IUP_DEFAULT;
}

/*--------------------------------------------------------------------*\
* Repaint the output canvas
\*--------------------------------------------------------------------*/
void main_repaint (Ihandle *dialog)
{
	Ihandle *canvas;

	/* repaint the canvas */
	canvas = IupGetAttribute(dialog, PAINTER_CANVAS);
	assert (canvas);
	cbRepaint(canvas, 0, 0);
}

/*--------------------------------------------------------------------*\
* Creates an image dialog with a canvas
* Input
*   title: title of the dialog
*   im: image to be displayed in the canvas
*   
* Output:
*   handle of the dialog
\*--------------------------------------------------------------------*/
static Ihandle* dlgImage (const char *title, p_image im, Ihandle *extrabox)
{
	Ihandle *d;
	Ihandle *iupcanvas;
	Ihandle *status, *type;
	cdCanvas *canvas;
	
	assert (im);

	/* Dialog with a canvas and a status bar */
	d = IupDialog(IupVbox(
			iupcanvas = IupCanvas("repaint_CB"), 
			type = IupLabel(""), 
      status = IupSetAttributes(IupLabel(""), "EXPAND=YES"),
			extrabox, 
			NULL));

	IupSetAttribute (d, PAINTER_IMAGE, (char*)im);
	IupSetAttribute (d, PAINTER_CANVAS, iupcanvas);
	IupSetAttribute (d, PAINTER_STATUS, status);
	IupSetAttribute (d, IUP_RESIZE, IUP_NO);
	IupSetAttribute (d, IUP_MAXBOX, IUP_NO);
	//IupSetAttribute (d, IUP_PARENTDIALOG, IupGetName(_main_dialog));
	IupSetfAttribute(d, IUP_TITLE, "%s", title);
	IupSetHandle(strdup(title), d);

	IupSetfAttribute(type, IUP_TITLE, "%d-channel [%dx%d]", im->type, im->width, im->height);
	IupSetfAttribute(iupcanvas, IUP_RASTERSIZE, "%dx%d", im->width+2, im->height+2);
	IupSetAttribute(iupcanvas, IUP_BUTTON_CB, "click_CB");
	IupSetAttribute(iupcanvas, IUP_CLOSE_CB, "close_CB");
	IupSetAttribute(iupcanvas, IUP_MOTION_CB, "motion_CB");
	IupSetFunction("click_CB", cbClick);
	IupSetFunction("repaint_CB", cbRepaint);
	IupSetFunction("motion_CB", cbMotion);
	IupMap(d);

	/* CD Canvas */
	canvas = cdCreateCanvas(CD_IUP, iupcanvas);
//	IupSetAttribute(iupcanvas, PAINTER_IMAGE, (char*)im);

	return d;
}

/*--------------------------------------------------------------------*\
* Applies a filter to the image
* Input
*   self: handle of the selected menu item
\*--------------------------------------------------------------------*/
int cbFilter (Ihandle *self)
{
	FILTER *f;
	Ihandle *canvas;
	char description[1024];
	char title[1024];
	p_image in, out;
	Ihandle *extrabox = NULL;

	sprintf (description, "");

	/* get canvas and image */
	canvas = IupGetAttribute(self, PAINTER_CANVAS);
	in = (p_image)IupGetAttribute(canvas, PAINTER_IMAGE);

	assert (canvas);
	assert (in);

	/* get filter function */
	f = (FILTER*)IupGetAttribute(self, PAINTER_FILTER);
	assert (f);

	/* create the output image */
	out = (p_image) malloc (sizeof(t_image));
	if (!out)
		return 0;

	/* bring the main dialog to top */
	IupShowXY (_main_dialog, 100, 100);

	/* execute the filter */
	if ((*f)(in, out, description, &extrabox))
	{
		/* create the title of the new image */
		sprintf (title, "%s of %s", description, IupGetAttribute(IupGetDialog(canvas), IUP_TITLE));

		/* opens a new image with the result */
		IupShowXY(dlgImage(title, out, extrabox), IUP_CENTER, IUP_CENTER);
	}
	else
	{
		IupMessage("Error", "Invalid parameters");
	}

	return IUP_DEFAULT;
}

/*--------------------------------------------------------------------*\
* Creates a filter item in the menu
* Input
*   title: text of the item
*   f: filter function to be called
* Output:
*   handle of the item
\*--------------------------------------------------------------------*/
static Ihandle* itemFilter (char *title, FILTER *f, FILTERCHECK *c)
{
	Ihandle *i;
	i = IupItem (title, "filter_CB");
	IupSetAttribute (i, PAINTER_FILTER, (char*)f);
	IupSetAttribute (i, PAINTER_FILTERCHECK, (char*)c);
	return i;
}

/*--------------------------------------------------------------------*\
* Creates the popup menu
* Output:
*   handle of the menu
\*--------------------------------------------------------------------*/
static Ihandle* mnPopup (void)
{
	Ihandle *menu;

	/* Popup menu of an image */
	menu = IupMenu(
		IupItem ("Open", "open_CB"),
		IupItem ("Save as...", "save_CB"),
		itemFilter ("Copy", fCopy, fTrue),
		IupSeparator(),
		itemFilter ("Grayscale", fGray, fTestRGB),
		itemFilter ("Red channel", fRed, fTestRGB),
		itemFilter ("Green channel", fGreen, fTestRGB),
		itemFilter ("Blue channel", fBlue, fTestRGB),
		IupSeparator(),
		itemFilter ("Scale...", fScale, fTrue),
		itemFilter ("Rotate...", fRotate, fTestMono),
		itemFilter ("Dither...", fDither, fTestMono),
		itemFilter ("Stroke area...", fStrokeArea, fTestRGB),
		itemFilter ("Color difference...", fColorDiff, fTestRGB),
		itemFilter ("Painterly rendering...", fPainter, fTestRGB),
		IupSeparator(),
		IupItem ("Close", "close_CB"),
		NULL
	);
	IupSetHandle(PAINTER_POPUP, menu);
	IupSetFunction("close_CB", cbClose);
	IupSetFunction("save_CB", cbSave);
	IupSetFunction("filter_CB", cbFilter);

	return menu;
}

/*--------------------------------------------------------------------*\
* Creates the main dialog
* Output:
*   handle of the dialog
\*--------------------------------------------------------------------*/

static Ihandle* dlgMain (void)
{
	Ihandle *d;
	Ihandle *menu;
	Ihandle *g;

	/* Main menu */
	menu = IupMenu(
		IupSubmenu("File", IupMenu(
			IupItem ("Open...", "open_CB"),
			IupSeparator(),
			IupItem ("Exit", "exit_CB"),
			NULL
		)),
		IupItem("3D Group", "about_CB"),
		NULL
	);
	IupSetHandle("mainmenu", menu);
	IupSetFunction ("open_CB", cbOpen);
	IupSetFunction ("exit_CB", cbExit);
	IupSetFunction ("about_CB", cbAbout);

	/* Main dialog */
	d = IupSetAttributes(IupDialog(
		IupHbox(
			IupSetAttributes(IupFill(), "SIZE=5"),
			IupVbox(
				IupSetAttributes(IupFill(), "SIZE=5"),
				g = IupSetAttributes(IupGauge(), "MARGIN=1, EXPAND=HORIZONTAL"),
				IupSetAttributes(IupFill(), "SIZE=5"),
				_status = IupSetAttributes(IupMultiLine(""), "EXPAND=YES, SIZE=200x200, READONLY=YES"),
				IupSetAttributes(IupFill(), "SIZE=5"),
				NULL
			),
			IupSetAttributes(IupFill(), "SIZE=5"),
			NULL
		)
	), "TITLE=Painter, MENU=mainmenu"
	);
	//IupSetHandle("PAINTER_MAIN_DIALOG", d);
	IupSetAttribute(d, IUP_CLOSE_CB, "exit_CB");

	gauge_open(g);

	return d;
}

int main (void)
{
	IupOpen();
	IupGaugeOpen();
	IupGetColorOpen();
	IupValOpen();
	warp_open(1024, 1024);
	
	IupShowXY(_main_dialog = dlgMain(), 100, 100);
	IupMainLoop();
	warp_close();

	IupClose();
}
