/*=========================================================================*\
* Benchmarking functions
* Diego Nehab
* 9/9/2000
\*=========================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "bench.h"

#ifdef WIN32
#include <windows.h>
#else
#include <sys/times.h>
#include <time.h>
#endif

/*=========================================================================*\
* Global definitions
\*=========================================================================*/
static int marks_per_second;

/*=========================================================================*\
* Auxiliary functions prototypes
\*=========================================================================*/
static int mark(void);

/*=========================================================================*\
* Exported function definitions
\*=========================================================================*/
/*-------------------------------------------------------------------------*\
* Initializes the module
\*-------------------------------------------------------------------------*/
void bench_open(void)
{
#ifdef WIN32
	marks_per_second = 1000;
#else
	marks_per_second = CLK_TCK;
#endif
}

/*-------------------------------------------------------------------------*\
* Marks current time.
* Input
*   timer: timer identifier or -1 to allocate new timer.
* Returns
*   timer identifier. -1 if no timer available or if error
\*-------------------------------------------------------------------------*/
int bench_mark(void)
{
	return mark();
}

/*-------------------------------------------------------------------------*\
* Prinst the same string repeatedly
* Input
*   timer: timer identifier
* Returns
*   number of seconds elapsed since last mark on timer, -1 in case of error
\*-------------------------------------------------------------------------*/
double bench_gettime(int t)
{
	return (double) (mark() - t) / (double) (marks_per_second);
}

/*-------------------------------------------------------------------------*\
* Frees all resources used by module
\*-------------------------------------------------------------------------*/
void bench_close(void)
{
	;
}

/*=========================================================================*\
* Auxiliary functions definitions
\*=========================================================================*/
#ifdef WIN32
static int mark(void)
{
	return GetTickCount();
}
#else
static int mark(void)
{
	struct tms b;
	return times(&b);
}
#endif

#ifdef BENCH_TEST
int main(void)
{
	int t;
	bench_open();
	t = bench_mark();
	while (1) 
		printf("%f\n", bench_gettime(t));
	bench_close();
	return 0;
}
#endif

