/*--------------------------------------------------------------------*\
* Error difusion dithering module
\*--------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "image.h"

#define DITHER_FLOYD

/*--------------------------------------------------------------------*\
* Performs error difusion in place, one image channel
* Input
*   in: image to be dithered
* Output:
*   0 if error, 1 otherwise
\*--------------------------------------------------------------------*/

#ifdef DITHER_FLOYD
/*--------------------------------------------------------------------*\
* Global variables
\*--------------------------------------------------------------------*/
static float *cur = NULL;
static float *nxt = NULL;
static int len = 0;

/*--------------------------------------------------------------------*\
* Swap the values of the two line pointers
\*--------------------------------------------------------------------*/
#define swap(a,b) \
	do { \
		float *tmp; \
		tmp = a; \
		a = b; \
		b = tmp; \
	} while (0)

/*--------------------------------------------------------------------*\
* Limit value to [0,255]
\*--------------------------------------------------------------------*/
#define choose(a) \
	((a < 0) ? 0 : ((a > 255) ? 255 : a))

/*--------------------------------------------------------------------*\
* Performs error difusion in place, one image channel
* Input
*   in: image to be dithered
* Output:
*   0 if error, 1 otherwise
\*--------------------------------------------------------------------*/
#define MAX(x,y) ((x)>(y)?(x):(y))
int dither_mono(p_image in, double s, double p)
{
	int x, y;
	int w, h;
	float error, value;
	float a = (float) ((s-1)/pow(255.0, p));
	unsigned char *buf, *buf_nxt;
	long skip;

	if (!in)
		return 0;

	srand(100);

	skip = in->row - in->width;
	w = in->width;
	h = in->height;
	buf = in->buffer;
	buf_nxt = buf + in->row;
	
	/* allocates working space */
	if (w > len) {
		cur = (float *) realloc(cur, w * sizeof(float));
		nxt = (float *) realloc(nxt, w * sizeof(float));
		len = w;
		if (!cur || !nxt) {
			if (cur) free(cur); cur = NULL;
			if (nxt) free(nxt); nxt = NULL;
			len = 0;
			return 0;
		}
	}

	/* inicializa o buffer de proxima linha */
	for (x = 0; x < w; x++)
		nxt[x] = MAX(1.0f/(a*(float)(pow(buf[x],p))+1), 0.5f);

	for (y = 0; y < h-1; y++) {

		/* next line becomes current line */
		swap(cur, nxt);

		/* copies next line to local buffer */
		nxt[0] = MAX(1.0f/(a*(float)(pow(*buf_nxt, p))+1), 0.5f);
		buf_nxt++;
		for (x = 1; x < w; x++) {
			nxt[x] = 1.0f/(a*(float)(pow(*buf_nxt, p))+1);
			buf_nxt++;
		}

		buf_nxt += skip;

		/* spread error */
		for (x = 0; x < w-1; x++) {
			value = cur[x] > 1.0f ? 1.0f : 0.0f;
			error = cur[x] - value;
			*buf++ = value > 0.0 ? 0 : 255;
			switch (rand() % 4) {
				case 0:
					nxt[x + 1] += error/16;
					nxt[x - 1] += 3*error/16;
					nxt[x]     += 5*error/16;
					cur[x + 1] += 7*error/16;
					break;
				case 1:
					nxt[x + 1] += 7*error/16;
					nxt[x - 1] += error/16;
					nxt[x]     += 3*error/16;
					cur[x + 1] += 5*error/16;
					break;
				case 2:
					nxt[x + 1] += 5*error/16;
					nxt[x - 1] += 7*error/16;
					nxt[x]     += error/16;
					cur[x + 1] += 3*error/16;
					break;
				case 3:
					nxt[x + 1] += 3*error/16;
					nxt[x - 1] += 5*error/16;
					nxt[x]     += 7*error/16;
					cur[x + 1] += error/16;
					break;
			}
		}

		*buf++ = cur[x] > 1.0 ? 0 : 255;

		buf += skip;
	}

	/* clip whole last line */
	for (x = 0; x < w; x++)
		buf[x] = nxt[x] > 1.0 ? 0 : 255;

	return 1;
}
#endif


