/*------------------------------------------------------------------*\
* Linked list implementation
* 3D group (Danilo, Diego and Diogo)
\*------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <assert.h>

#include "list.h"

/*------------------------------------------------------------------*\
* Tipos de dados usados.
\*------------------------------------------------------------------*/
typedef struct t_list {
	int xc, yc;
	float w, l;
	uchar rgb[3];
	float theta;
	int level;
	p_list next;
} t_list;

/*------------------------------------------------------------------*\
* Prototipo das funccoes internas
\*------------------------------------------------------------------*/
static p_list list_newnode(int xc, int yc, float w, float l, float theta, uchar *rgb, int level);
static void list_split(p_list h, p_list *l1, p_list *l2);
static void list_merge(p_list *h, p_list l1, p_list l2);

/*------------------------------------------------------------------*\
* Cria uma nova lista
\*------------------------------------------------------------------*/
p_list list_create(void)
{
	return NULL;
}

/*------------------------------------------------------------------*\
* Enumera os elementos da lista
\*------------------------------------------------------------------*/
p_list list_enum(p_list p, int *xc, int *yc, int *w, int *l, float *theta, uchar *rgb, int *level)
{
	if (p) {
		if (xc) *xc = p->xc;
		if (yc) *yc = p->yc;
		if (w) *w = (int)p->w;
		if (l) *l = (int)p->l;
		if (theta) *theta = p->theta;
		if (rgb) {
			rgb[0] = p->rgb[0];
			rgb[1] = p->rgb[1];
			rgb[2] = p->rgb[2];
		}
		if (level) *level = p->level;
		return p->next;
	}
	return NULL;
}

/*------------------------------------------------------------------*\
* Insere um elemento no inicio da lista. 
\*------------------------------------------------------------------*/
void list_prepend(p_list *h, int xc, int yc, float w, float l, float theta, uchar *rgb, int level)
{
	p_list n = list_newnode(xc, yc, w, l, theta, rgb, level);
	n->next = *h;
	*h = n;
}

/*------------------------------------------------------------------*\
* Ordena a lista pelo metodo merge sort.
\*------------------------------------------------------------------*/
void list_mergesort(p_list *h)
{
	p_list l1, l2;
	
	if (*h && (*h)->next) {
		list_split(*h, &l1, &l2);
		list_mergesort(&l1); 
		list_mergesort(&l2);	
		list_merge(h, l1, l2);
	}
}

/*------------------------------------------------------------------*\
* Libera a memoria ocupada pela lista.
\*------------------------------------------------------------------*/
void list_kill(p_list *h)
{
	p_list p1, p2;
	assert(h);
	p1 = *h;
	while (p1) {
		p2 = p1->next;
		free(p1);
		p1 = p2;
	}
	*h = NULL;
}

/*------------------------------------------------------------------*\
* Imprime a lista
\*------------------------------------------------------------------*/
void list_print(p_list p)
{
	while (p) {
		printf("(%d, %d), (%.4f, %.4f), %f, (%d %d %d), %d\n", p->xc, p->yc, p->w, 
			p->l, p->theta, p->rgb[0], p->rgb[1], p->rgb[2], p->level);
		p = p->next;
	}
}


/*------------------------------------------------------------------*\
* Cria um novo elemento para a lista.
\*------------------------------------------------------------------*/
static p_list list_newnode(int xc, int yc, float w, float l, float theta, uchar *rgb, int level) 
{
	p_list t;
	t = (p_list) malloc(sizeof(t_list));
	assert(t != NULL);
	if (t != NULL) {
		t->xc = xc;
		t->yc = yc;
		t->w = w;
		t->l = l;
		t->theta = theta;
		t->rgb[0] = rgb[0];
		t->rgb[1] = rgb[1];
		t->rgb[2] = rgb[2];
		t->level = level;
		t->next = NULL;
	}
	return t;
}

/*------------------------------------------------------------------*\
* Junta duas listas ordenadas em uma lista ordenada com todos os elementos.
* As duas listas anteriores sao destruidas no processo. 
\*------------------------------------------------------------------*/
static void list_merge(p_list *h, p_list l1, p_list l2)
{
	float cmp;
	while (l1 != NULL && l2 != NULL) {
		cmp = (l1->w*l1->l) - (l2->w*l2->l);
		if (cmp > 0) {
			(*h) = l1;
			l1 = l1->next;
		}
		else {
			(*h) = l2;
			l2 = l2->next;
		}
		h = &(*h)->next;
	}
	if (l1 != NULL) 
		*h = l1;
	else if (l2 != NULL) 
		*h = l2;
	else 
		*h = NULL;
}

/*------------------------------------------------------------------*\
* Separa uma lista em duas listas com a metade do tamanho. A lista 
* original eh destruida no processo.
\*------------------------------------------------------------------*/
static void list_split(p_list h, p_list *l1, p_list *l2)
{
	*l1 = NULL;
	*l2 = NULL;
	while (1) {
		*l1 = h;
		h = h->next;
		if (h == NULL) {
			*l2 = NULL;
			break;
		}
		l1 = &((*l1)->next);
		*l2 = h;
		h = h->next;
		if (h == NULL) {
			*l1 = NULL;
			break;
		}
		l2 = &((*l2)->next);
	}
}
