
#include <stdlib.h>
#include <math.h>

#include "image.h"
#include "painter.h"
#include "gauge.h"


static double d02 = 150*150;
//static double Xn = 0.9510000000000001, Yn = 0.9999999999999999, Zn = 1.089;
//static double Xn = 0.980722, Yn = 1.0, Zn = 1.18224;
#define Xn 0.980722
#define Yn 1.0
#define Zn 1.18224

#define COMDEN Xn + 15*Yn + 3*Zn
#define uln 4*Xn / COMDEN
#define vln 9*Yn / COMDEN

#define iround(x) (int) (x + 0.5)

/*------------------------------------------------------------------*\
 * Calculates the euclidian color distance in RGB space             *
\*------------------------------------------------------------------*/
static double color_distance2(uchar *bgr1, uchar *bgr2)
{
   int dr, dg, db;

   dr = (int) *bgr1++ - (int) *bgr2++;
   dg = (int) *bgr1++ - (int) *bgr2++;
   db = (int) *bgr1 - (int) *bgr2;

   return dr * dr + dg * dg + db * db;
}


static void xyz2luv(double x, double y, double z, double *L, double *u, double *v)
{
	double ry;
	double ul, vl;//, uln, vln;
	double den;

	ry = y/Yn;

	if (ry > 0.008856)
		*L = 116*(pow(ry,1/3));
	else
		*L = 903.3*ry;

	den = x + 15*y + 3*z;

	ul = 4*x / den;
	vl = 9*y / den;

//	den = Xn + 15*Yn + 3*Zn;
//	uln = 4*Xn / den;
//	vln = 9*Yn / den;

	*u = 13*(*L) * (ul-uln);
	*v = 13*(*L) * (vl-vln);
}


static double color_distance_luv(uchar *bgr1, uchar *bgr2)
{
	double x1, y1, z1;
	double x2, y2, z2;

	double L1, v1, u1;
	double L2, v2, u2;

	double dL, du, dv;

	x1 = 0.607*(*bgr1+2) + 0.174*(*bgr1+1) + 0.200*(*bgr1);
	y1 = 0.299*(*bgr1+2) + 0.587*(*bgr1+1) + 0.114*(*bgr1);
	z1 = 0.000*(*bgr1+2) + 0.066*(*bgr1+1) + 1.116*(*bgr1);

	x2 = 0.607*(*bgr2+2) + 0.174*(*bgr2+1) + 0.200*(*bgr2);
	y2 = 0.299*(*bgr2+2) + 0.587*(*bgr2+1) + 0.114*(*bgr2);
	z2 = 0.000*(*bgr2+2) + 0.066*(*bgr2+1) + 1.116*(*bgr2);

	xyz2luv(x1, y1, z1, &L1, &u1, &v1);
	xyz2luv(x2, y2, z2, &L2, &u2, &v2);

	dL = L1-L2;
	du = u1-u2;
	dv = v1-v2;

	return dL*dL + du*du + dv*dv;
}


static double mom_I(uchar *bgrxy, uchar *bgr)
{
   double d2, r2;

   d2 = color_distance2 (bgrxy, bgr);
//   d2 = color_distance_luv (bgrxy, bgr);

   if (d2 >= d02)
      return 0.0;

   r2 = d2 / d02;

   return ((1 - r2) * (1 - r2));
}

static double mom_00(const p_image in, uchar *bgr)
{
   int i, j;
   double m;
   uchar *p;
   long skip = (in->row - in->width)*in->type;

   m = 0.0;
   p = in->buffer;
   for (i = 0; i < in->height; i++) {
      for (j = 0; j < in->width; j++) {
         m += mom_I (p, bgr);
         p += in->type;
      }
      p += skip;
   }

   return m;
}

int painter_strokearea(const p_image in, int s, p_image out)
{
	int i, j;
	double m;
	uchar *input, *output;
	long iskip, oskip; 
	t_image piece;

	if (!image_create(in->width, in->height, IMAGE_MONO, out))
		return 0;

	gauge_setmax(in->width*in->height);
	gauge_start();

	output = out->buffer;
	input = in->buffer;
	iskip = (in->row - in->width) * in->type; 
	oskip = (out->row - out->width) * out->type; 
 
	for (i = 0; i < in->height; i++) {
		for (j = 0; j < in->width; j++) {
			image_crop(in, j, i, s, s, &piece);
			m = mom_00(&piece, input);
			*output = (uchar) (255.0 * m / (piece.width*piece.height));
			output++;
			input += in->type;
		}
		gauge_increase(in->width);
		gauge_print();
		
		output += oskip;
		input += iskip;
	}
	gauge_end();
	return 1;
}


int painter_colordiff(const p_image in, uchar *bgr, p_image out)
{
	int i, j;
	double m;
	uchar *bufin, *bufout;
	long skipin, skipout;

	bufin = in->buffer;
	bufout = out->buffer;
	skipin = (in->row - in->width) * in->type;
	skipout = (out->row - out->width);

	for (i = 0; i < in->height; i++) {
		for (j = 0; j < in->width; j++) {
			m = mom_I(bufin, bgr);
			*(bufout++) = (uchar) (m * 255);
			bufin += in->type;
		}
		bufin += skipin;
		bufout += skipout;
	}

	return 1;
}


static int painter_moments(const p_image in, uchar *bgr, double *m00, double *m01, double *m10, double *m11, double *m02, double *m20, p_image I)
{
	int y, x;
	uchar *bufI;
	long skip;

	painter_colordiff(in, bgr, I);
	bufI = I->buffer;
 	skip = (I->row-I->width)*I->type;
	*m00 = *m01 = *m10 = *m11 = *m02 = *m20 = 0;

	for (y = 1; y <= I->height; y++) {
		for (x = 1; x <= I->width; x++) {
			*m00 += (*bufI);
			*m01 += y * (*bufI);
			*m10 += x * (*bufI);
			*m11 += y * x * (*bufI);
			*m02 += y * y * (*bufI);
			*m20 += x * x * (*bufI);
			bufI++;
		}
		bufI += skip;
	}

	return 1;
}


static void coloravg(const p_image in, uchar *xybgr)
{
	int i, j;
	int skip;
	int area;
	int bgr[3];
	uchar *bufin = in->buffer;

	bgr[0] = bgr[1] = bgr[2] = 0;
 	skip = (in->row-in->width)*in->type;

	for (i = 0; i < in->height; i++) {
		for (j = 0; j < in->width; j++) {
			bgr[0] += *bufin++;
			bgr[1] += *bufin++;
			bgr[2] += *bufin++;
		}
		bufin += skip;
	}

	area = in->width * in->height;

	*xybgr = bgr[0] / area;
	xybgr++;
	*xybgr = bgr[1] / area;
	xybgr++;
	*xybgr = bgr[2] / area;
}

int painter_strokepar(const p_image in, int x, int y, int s, int *xc, int *yc, double *theta, float *w, float *l, uchar *xybgr, p_image I)
{
	t_image crop;
	double m00, m01, m10, m11, m02, m20;
	double a, b, c;
	double tempval;
	double dw, dxc, dyc;
	uchar *color;

	image_crop(in, x, y, s, s, &crop);

	// coloravg(&crop, xybgr);
	color = in->buffer + (in->row*y + x) * in->type;
	xybgr[0] = color[0];
	xybgr[1] = color[1];
	xybgr[2] = color[2];

	painter_moments(&crop, xybgr, &m00, &m01, &m10, &m11, &m02, &m20, I);

	dxc = m10 / m00;
	dyc = m01 / m00;
	a = (m20 / m00) - (double)((dxc)*(dxc));
	b = 2 * (m11 / m00 - (double)((dxc)*(dyc)));
	c = (m02 / m00) - (double)((dyc)*(dyc));
	*theta = atan2(b, (a-c)) / 2;
	tempval = sqrt(b*b + (a-c)*(a-c));
	dw = sqrt(6 * (a+c - tempval));
	*w = (float)(sqrt(6 * (a+c - tempval)));
	*l = (float)(sqrt(6 * (a+c + tempval)));
	*xc = x + iround(dxc - s/2);
	*yc = y + iround(dyc - s/2);

	return 1;
}
